#include <string.h>
#include "op5license.h"
#include "rsa.h"

char license_buffer[BUFSIZE];

typedef struct pair
{
    char *key;
    char *value;
} pair_t;

static int sort_by_key(const void *x, const void *y) 
{
	pair_t **xx = (pair_t**)x;
	pair_t **yy = (pair_t**)y;
	return strcmp((*xx)->key, (*yy)->key);
}

static unsigned char *hexify(unsigned char *data, unsigned int len)
{
	int i, j = 0;
	unsigned char *ret;

	if (!(ret = malloc((len * 2) + 1)))
		return NULL;

	for (i = 0; i < len; i++) {
		const char hex[] = "0123456789abcdef";

		ret[j++] = hex[data[i] >> 4];
		ret[j++] = hex[data[i] & 0xf];
	}

	/* nul-terminate the buffer */
	ret[j] = 0;

	return ret;
}

unsigned char *calculate_digest(unsigned char *input)
{
	EVP_MD_CTX mdctx;
	const EVP_MD *md;
	unsigned char md_value[EVP_MAX_MD_SIZE];
	unsigned int md_len;

	md = EVP_sha1();
	EVP_MD_CTX_init(&mdctx);
	EVP_DigestInit_ex(&mdctx, md, NULL);
	EVP_DigestUpdate(&mdctx, input, strlen((char *)input));
	EVP_DigestFinal_ex(&mdctx, md_value, &md_len);
	EVP_MD_CTX_cleanup(&mdctx);

	return hexify(md_value, md_len);
}

char *license_to_string(ezxml_t license_node)
{
	ezxml_t cur_node = NULL;
	int buflen = 0;
	
	for (cur_node = license_node->child; cur_node; cur_node = cur_node->ordered)
	{
		if(!strcmp(cur_node->name, "item"))
		{
			char *name, *value;
			int len;

			name = (char *)ezxml_attr(cur_node, "name");
			value = (char *)ezxml_attr(cur_node, "value");

			if (BUFSIZE <= buflen + strlen(name) + strlen(value) + 4) {
				printf("License buffer exceeded. Aborting.\n");
				exit(2);
			}

			len = strlen(name);
			memcpy(&license_buffer[buflen], name, len);
			buflen += len;
			license_buffer[buflen++] = ':';
			license_buffer[buflen++] = ' ';

			len = strlen(value);
			memcpy(&license_buffer[buflen], value, len);
			buflen += len;
			license_buffer[buflen++] = ';';
		}
	}
	return license_buffer;
}

int parse_lic_v2(ezxml_t license_node, char *preffix, pair_t **result)
{
	int i, s,  count = 0;
	ezxml_t cur_node = NULL;
	const char *interesting_attrs[] = {"value", "IssueDate", "ExpireDate", "lifetime", "Hosts"};

	for (cur_node = license_node->child; cur_node; cur_node = cur_node->ordered)
	{
	
		if(strcmp(cur_node->name, "item") == 0)
		{
		
			pair_t *p = malloc(sizeof(pair_t));
			char *name, *key, *value;

			name = (char *)ezxml_attr(cur_node, "name");

			key = malloc(strlen(preffix)+strlen(name)+2);
			sprintf(key, "%s/%s", preffix, name);

			for(s = 0, i = 0; i < 3; i++)
			{
				if(ezxml_attr(cur_node, interesting_attrs[i]))
					s += strlen(interesting_attrs[i]) + strlen(ezxml_attr(cur_node, interesting_attrs[i])) + 2;
			}
			value = malloc(s + 1);
			for(s = 0, i = 0; i < 3; i++)
			{
				if(ezxml_attr(cur_node, interesting_attrs[i]))
				{
					sprintf(value + s, "%s=%s|", interesting_attrs[i], ezxml_attr(cur_node, interesting_attrs[i]));
					s += strlen(interesting_attrs[i]) + strlen(ezxml_attr(cur_node, interesting_attrs[i])) + 2;
				}
			}

			p->key = key;
			p->value = value;

			result[count++] = p;
		}
		else
		{
			char *pref = malloc(strlen(preffix)+strlen(cur_node->name)+2);

			sprintf(pref, "%s/%s", preffix, cur_node->name);

			count += parse_lic_v2(cur_node, pref, result+count);
		}
	}
	
	return count;
}

char *license_to_string_v2(ezxml_t license_node)
{
	int buflen = 0;
	int i, len, count;
	pair_t **items = malloc(MAX_ITEMS*sizeof(pair_t*));
	count = parse_lic_v2(license_node, "", items);
	
	//printf("real sixe: %d", count);
	qsort(items, count, sizeof(pair_t*), sort_by_key);

	for(i=0; i<count; i++)
	{
		if(items[i] != NULL)
		{
			
		
			if (BUFSIZE <= buflen + strlen(items[i]->key) + strlen(items[i]->value) + 4) 
			{
				printf("License buffer exceeded. Aborting.\n");
				exit(2);
			}

			len = strlen(items[i]->key);
			memcpy(&license_buffer[buflen], items[i]->key, len);
			buflen += len;

			license_buffer[buflen++] = ':';

			len = strlen(items[i]->value);
			memcpy(&license_buffer[buflen], items[i]->value, len);
			buflen += len;

			license_buffer[buflen++] = '\n';
		}
	}

	return license_buffer;
}

