#include <string.h>
#include <openssl/evp.h>
#include <openssl/rsa.h>
#include <openssl/bio.h>
#include <openssl/pem.h>
#include <openssl/err.h>
#include <openssl/bn.h>
#include <op5/license_logging.h>

RSA *init_public_key(char *n, char *e)
{
	RSA *key = RSA_new();
	if(BN_hex2bn(&key->n, n) < 1)
	{
		log_err("BN_hex2bn() failed; could not set n value of public key. Aborting.\n");
		return NULL;
	}

	if(BN_hex2bn(&key->e, e) < 1)
	{
		log_err("BN_hex2bn() failed; could not set e value of public key. Aborting.\n");
		return NULL;
	}

	return(key);
}

unsigned char *public_decrypt(unsigned char *input, RSA *key)
{
	unsigned char *output = NULL;
	int keylen = RSA_size(key);
	int outlen;

	/* from the RSA_public_decrypt() man-page:
	 * int RSA_public_decrypt(int flen, unsigned char *from,
	 *            unsigned char *to, RSA *rsa, int padding);
	 *
	 * RSA_public_decrypt() recovers the message digest from the
	 * flen bytes long signature at from using the signer’s public
	 * key rsa. to must point to a memory section large enough to
	 * hold the message digest (which is smaller than RSA_size(rsa) - 11).
	 * padding is the padding mode that was used to sign the data.
	 *
	 * in short: We won't need more than RSA_size(key) - 10
	 * bytes to store the decrypted string and a terminating
	 * NUL byte.
	 */
	output = malloc(keylen - 10);
	outlen = RSA_public_decrypt(keylen, input, output, key, RSA_PKCS1_PADDING);

	if (outlen < 0) {
		printf("RSA_public_decrypt() failed: %s\n", ERR_error_string(ERR_get_error(), NULL));
		return NULL;
	}

	if (outlen > keylen - 10)
		return NULL;

	output[outlen] = '\0';

	return output;
}
